﻿using System;
using System.Collections.Generic;
using System.Runtime.Serialization;
using HIPS.CommonSchemas;

namespace HIPS.PcehrSchemas
{
    /// <summary>
    /// This object is used to return the view and possible list of attachments from the PCEHR system.
    /// </summary>

    [KnownType(typeof(ViewResponse))]
    [Serializable]
    [DataContract]
    public class ViewResponse
    {
        #region Private Members

        private byte[] document;
        private string fileName;
        private string templateId;
        private string mimeType;
        private HipsResponse hipsResponse;
        private IList<Attachment> attachments = new List<Attachment>();

        #endregion Private Members

        #region Properties

        /// <summary>
        /// Gets or Sets the Document
        /// </summary>
        /// <value>
        /// The document
        /// </value>
        [DataMember]
        public byte[] Document
        {
            get { return document; }
            set { document = value; }
        }

        /// <summary>
        /// Gets or Sets the FileName
        /// </summary>
        /// <value>
        /// The fileName
        /// </value>
        [DataMember]
        public String FileName
        {
            get { return fileName; }
            set { fileName = value; }
        }

        /// <summary>
        /// Gets or Sets the DocumentUniqueId
        /// </summary>
        /// <value>
        /// The documentUniqueID
        /// </value>
        [DataMember]
        public String TemplateId
        {
            get { return templateId; }
            set { templateId = value; }
        }

        /// <summary>
        /// Gets or Sets the MimeType
        /// </summary>
        /// <value>
        /// The mimeType
        /// </value>
        [DataMember]
        public String MimeType
        {
            get { return mimeType; }
            set { mimeType = value; }
        }

        /// <summary>
        /// Gets or sets the HipsResponse, which contains information about success, warnings or failure.
        /// </summary>
        /// <value>
        /// The HipsResponse
        /// </value>
        [DataMember]
        public HipsResponse HipsResponse
        {
            get { return hipsResponse; }
            set { hipsResponse = value; }
        }

        /// <summary>
        /// Gets or Sets the Attachments
        /// </summary>
        /// <value>
        /// The attachments
        /// </value>
        [DataMember]
        public IList<Attachment> Attachments
        {
            get { return attachments; }
            set { attachments = value; }
        }

        #endregion Properties
    }
}